package com.casic.missiles.service.meter;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.casic.missiles.dto.BaseApprovalSubmitRequest;
import com.casic.missiles.dto.ReturnDTO;
import com.casic.missiles.dto.flowable.ApprovalDeleteRequest;
import com.casic.missiles.dto.meter.*;
import com.casic.missiles.model.meter.MeterTraceSupplier;

import java.util.List;

/**
 * <p>
 * 计量管理-溯源供方-溯源供方表(计量计量单位的设备的单位) 服务类
 * </p>
 *
 * @author wangpeng
 * @since 2022-11-28
 */
public interface IMeterTraceSupplierService extends IService<MeterTraceSupplier> {

    Page<MeterTraceSupplier> listPage(Page<MeterTraceSupplier> page, TraceSupplierListRequest request);

    ReturnDTO updateTraceSupplier(MeterTraceSupplier traceSupplier);

    ReturnDTO deleteTraceSupplier(Long id);

    ReturnDTO deleteBatchTraceSupplier(List<Long> ids);

    ReturnDTO<TraceSupplierDetailResponse> traceSupplierDetail(Long id);

    List<MeterTraceSupplier> list(TraceSupplierListRequest request);

    ReturnDTO<TraceSupplierSaveResponse> saveTraceSupplier(MeterTraceSupplier traceSupplier);

    ReturnDTO addTraceSupplierList(List<MeterTraceSupplier> list) throws Exception;

    ReturnDTO draftUpdate(MeterTraceSupplier traceSupplier);

    ReturnDTO submitTraceSupplier(BaseApprovalSubmitRequest request);

    List<TraceSupplierApprovalListResponse> approvalListPage(Page<TraceSupplierApprovalListResponse> page, TraceSupplierApprovalListRequest request);

    ReturnDTO approvalDelete(ApprovalDeleteRequest request);

    ReturnDTO failUpdate(MeterTraceSupplier traceSupplier);
}
